from pymongo import MongoClient
client=MongoClient('mongodb+srv://sailor:O9d4*eBJkizk@cluster0-7aeh1.azure.mongodb.net/test?retryWrites=true&w=majority')
marketdb=client.market_test
import socket,requests
from multiprocessing import Pool
from requests.exceptions import ConnectionError
from itertools import repeat
import json
local_ip_address=socket.gethostbyname(socket.gethostname())
users=marketdb.users

def getUsers():
    output=[]
    cursor=users.find({})
    for user in cursor:
        output.append(user)
    return output

def getIPList():
    return [user['ip'] for user in getUsers()]

def emit(endpoint,message,ip=None,port=2002):
    url=None
    if ip is None:
        url='http://{}'.format(endpoint)
    else:
        url='http://{}:{}/{}'.format(ip,str(port),endpoint)
    try:
        return requests.post(url,data={'message': message},timeout=5).text
    except ConnectionError as e:
        print("Could not find destination", endpoint,ip)
        return None
        

def broadcast(endpoint,message):
    ips=getIPList()
    res=None
    with Pool(len(ips)) as pool:
        args=zip(repeat(endpoint),repeat(message),ips)
        res=pool.starmap(emit,args)
        
def getProfile():
    user=users.find_one({'ip': local_ip_address})
    if user is not None:
        try:
            from Business import Business
        except:
            from mural.Business import Business
        return Business.load(user['name'])
    else: return None

if __name__=='__main__':
    color={
        'x': 0,
        'y': 0,
        'r': 0,
        'g': 0,
        'b': 0,
    }
    broadcast('recievePoint',json.dumps(color))



try:
    from Market import Market
    from Business import Business
except:
    from mural.Market import Market
    from mural.Business import Business
import random
import time
sc=Market('Santa Cruz',(1,3))
sf=Market('San Francisco',(-2,7))
rwc=Market('Redwood City',(10,-8))


nike=Business('Nike',sc,300,30)
adidas=Business('Adidas',sf,200,20)
asic=Business('Asic',sf,200,20)
reebok=Business('Reebok',sf,200,20)
sketchers=Business('Sketchers',rwc,200,20)
puma=Business('Puma',rwc,200,20)
vans=Business('Vans',sc,200,20)

sketchers.setAskPrice(1)
sketchers.setBidPrice(30)

print("All the businesses are setup.\n")
time.sleep(4)
rwc.display()

time.sleep(4)
print("Sending out convoys\n")
convoy=puma.createConvoy(cost=70,money=30,quantity=5)
convoy.setAskPrice(1)
convoy.gotoMarket(sc,[(vans,2),(nike,3)])
time.sleep(4)
convoy2=vans.createConvoy(cost=10,money=5,quantity=1)
convoy2.gotoMarket(sf,[(asic,1),(reebok,1)])
time.sleep(4)
print("\nAttempting to make a trasaction with a convoy while they are traveling.")
convoy.sell(nike,3)

time.sleep(4)
rwc.display()

time.sleep(4)
print("\nDelaying for 15 secconds. While convoys travel.")
time.sleep(15)

print("\nAttempting to make a convoy transact.")
convoy.sell(nike,3)

time.sleep(4)
print("\nDelaying for 20 secconds. While convoys travel home.")
time.sleep(10)

rwc.display()






from Market import Market
from Business import Business
import random
myMarket=Market('valeria')
myBiz=Business('Nike')
urBiz=Business('Addidas')

myMarket.addBusiness(myBiz)
myMarket.addBusiness(urBiz)

print( "My market is named %s" % myMarket.name )
print( "%s containes the businesses: %s\n" % (myMarket.name, myMarket.businesses) )

while not myBiz.sell(urBiz, 5):
    print("My ask price $%.02f"%myBiz.askPrice, " ur bid price $%.02f"%urBiz.bidPrice)
    myBiz.setAskPrice(myBiz.askPrice-random.random())
    urBiz.setBidPrice(urBiz.bidPrice+random.random())

for business in myMarket.businesses:
    print("The business %s:"%business)
    print("\tMoney: $%.02f"%business.money)
    print("\tQuantity: %d"%business.quantity)
    print("\n")




try:
    from Saveable import Saveable
    from db import marketdb
except:
    from mural.Saveable import Saveable
    from mural.db import marketdb
    
_commodity=marketdb.commodity
class Stock:
    def __init__(self,commodity,quantity=0,buyPrice=1,sellPrice=1e3):
        if issubclass(type(commodity),str):
            commodity=Commodity.load(commodity.title())
        self.name=commodity.name
        self.commodity=commodity
        self.buyPrice=buyPrice
        self.sellPrice=sellPrice
        self.quantity=quantity

    def load(data):
        return Stock(Commodity.load(data['name']),data['quantity'],data['buy'],data['sell'])
    def data(self):
        return {
            'name': self.name,
            'buy': self.buyPrice,
            'sell': self.sellPrice,
            'quantity': self.quantity,
        }
        
class Commodity(Saveable):
    def __init__(self,name,_id=None):
        self.name=name
        self.save()
        self._id=_id
    def __repr__(self):
        return "Commodity %s"%self.name
    def load(name):
        data=Saveable.load(name,_commodity)
        return Commodity(data['name'],data['_id'])
    def data(self):
        return {'name': self.name }
    def save(self):
        super(Commodity,self).save(_commodity)
    def all():
        collect=[]
        cursor=_commodity.find({})
        for commodity in cursor:
            collect.append(Commodity(commodity['name']))
        return collect
    def getStock(self):
        return Stock(self)
        



from math import sqrt
try:
    from db import marketdb
    from Saveable import Saveable
except:
    from mural.db import marketdb
    from mural.Saveable import Saveable
_market=marketdb.market
class Market(Saveable):
    def __init__(self,name,position=(0,0),exports=[],_id=None):
        super(Market,self).__init__()
        self._id=_id
        if self._id is None:
            self._id=Saveable.getId(name,_market)
        self.name=name
        self.position=position
        self.exports=exports
        self.businesses=[]

    def getMainExport(self):
        return self.exports[0]
    def load(d,deep=True):
        d=Saveable.load(d,_market)
        market=Market(d['name'],d['position'],d['exports'],d['_id'])
        if deep:
            try:
                from Business import Business
            except:
                from mural.Business import Business
            for business in d['businesses']:
                Business.load(business,market)
        return market
    
    def save(self):
        super(Market,self).save(_market)
        for b in self.businesses: b.save()
        

    def data(self):
        return {
            'name': self.name,
            'businesses': [b._id for b in self.businesses],
            'position': self.position,
            'exports': [e.name for e in self.exports],
        }
    def __repr__(self):
        return self.name
    
    def display(self):
        for business in self.businesses:
            business.display()

    def getDistance(self,market):
        x=market.position[0]-self.position[0]
        y=market.position[1]-self.position[1]
        return sqrt(x**2 + y**2)
    
    def addBusiness(self,business):
        self.businesses.append(business)
        
    def all():
        output=[]
        cursor=_market.find({})
        for market in cursor:
            output.append(market)
        return output
    






import setuptools

setuptools.setup(
    name="mural",
    version="0.0.11",
    author="William Wyatt",
    author_email="wwyatt@ucsc.edu",
    description="Market game",
    long_description="Market Game.",
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    url="https://github.com/Tsangares/market",
    include_package_data=True,
    scripts=[
        'mural/bin/business',
        'mural/bin/market',
        'mural/bin/mine',
        'mural/bin/transact',
        'mural/bin/update',
        'mural/bin/oracle',
        'mural/bin/convoy',
        'mural/bin/setAccount',
        'mural/bin/net'
    ],
    install_requires=[
        "requests==2.22.0",
	"aniso8601==7.0.0",
	"bleach==3.1.0",
	"certifi==2019.6.16",
	"chardet==3.0.4",
	"Click==7.0",
	"dnspython==1.16.0",
	"docutils==0.14",
	"Flask==1.0.3",
	"Flask-PyMongo==2.3.0",
	"Flask-RESTful==0.3.7",
	"Flask-SocketIO==4.1.0",
	"idna==2.8",
	"itsdangerous==1.1.0",
	"Jinja2==2.10.1",
	"jsonify==0.5",
	"MarkupSafe==1.1.1",
	"numpy==1.16.4",
	"pkginfo==1.5.0.1",
	"prompt-toolkit>=1.0.14",
	"Pygments==2.4.2",
	"PyInquirer>=1.0.3",
	"pymongo==3.8.0",
	"python-engineio==3.8.2.post1",
	"python-socketio==4.2.0",
	"pytz==2019.1",
	"PyYAML==5.1.1",
	"readme-renderer==24.0",
	"regex==2019.6.8",
	"requests-toolbelt==0.9.1",
	"six==1.12.0",
	"tqdm==4.32.2",
	"twine==1.13.0",
	"urllib3==1.25.3",
	"wcwidth==0.1.7",
	"webencodings==0.5.1",
	"Werkzeug==0.15.4",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
)
